----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    20:25:31 03/25/2011 
-- Design Name: 
-- Module Name:    Buttons_VHDL - Verhalten 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

	--------------------------------------------
	-- Beschreibung :
	--
	-- hier werden die Buttons und Schalter
	-- mit dem Clock syncronisiert
	--
	-- weil die Taster prellen
	-- werden sie nur alle 100ms eingelesen
	-- sonst kommt es zu anzeigefehlern
	--
	-- damit die Read/Write Funktionen
	-- bei gedruecktem Button nicht
	-- staendig aufgerufen werden,
	-- wird aus allen 4 Buttons
	-- ein OnClick-Eregnis gemacht
	--------------------------------------------
	
entity Buttons_VHDL is

	--------------------------------------------
	-- Port Deklerationen
	--------------------------------------------
	port (
		clk_in : in std_logic;
		BTN_IN : in std_logic_vector(3 downto 0);
		SW_IN : in std_logic_vector(3 downto 0);
		button_out : out std_logic_vector(7 downto 0);
		taster_out : out std_logic_vector(3 downto 0)
	);
		
end Buttons_VHDL;

architecture Verhalten of Buttons_VHDL is

	--------------------------------------------
	-- Interne Signale
	--------------------------------------------
	signal v_button : std_logic_vector(7 downto 0) := (others => '0');
	signal v_btn_old : std_logic_vector(3 downto 0):= (others => '0');	
	signal v_btn_tic : std_logic_vector(3 downto 0):= (others => '0');	
	
	-- verzoegerung fr die tasten, weil sie prellen
	constant bnt_sleep : integer := 13300000; -- Zeit = 100ms
	signal v_counter :  natural range 0 to bnt_sleep := bnt_sleep; 	

begin

	-----------------------------------------
	-- Einclocken von Asyncronen Signalen
	-----------------------------------------
	P_ASYNC : process(clk_in)
	begin
		if falling_edge(clk_in) then
			if v_counter = 0 then
			   -- entprellte tasten
				v_counter <= bnt_sleep;
				v_button(0) <= BTN_IN(0);
				v_button(1) <= BTN_IN(1);
				v_button(2) <= BTN_IN(2);
				v_button(3) <= BTN_IN(3);
				v_button(4) <= SW_IN(0);
				v_button(5) <= SW_IN(1);
				v_button(6) <= SW_IN(2);
				v_button(7) <= SW_IN(3);
			else
				v_counter <= v_counter -1;			
			end if;								
		end if;
	end process P_ASYNC;
	
	button_out <= v_button;
	
	-----------------------------------------
	-- Aus den Tastern jeweils ein
	-- einzelnes 1Clock Signal machen,
	-- bis der Taser wieder verlassen ist
	-- (damit jede Funktion nur einmal ausgefhrt wird)
	-----------------------------------------
	P_Buttons : process(clk_in)
	begin
		if falling_edge(clk_in) then
			-- east
			v_btn_tic(0) <= '0';
			if v_button(0)='1' and v_btn_old(0)='0' then
				v_btn_old(0) <= '1';
				v_btn_tic(0) <= '1';
			elsif v_button(0)='0' and v_btn_old(0)='1' then
				v_btn_old(0) <= '0';
			end if;
			-- north
			v_btn_tic(1) <= '0';
			if v_button(1)='1' and v_btn_old(1)='0' then
				v_btn_old(1) <= '1';
				v_btn_tic(1) <= '1';
			elsif v_button(1)='0' and v_btn_old(1)='1' then
				v_btn_old(1) <= '0';
			end if;
			-- south
			v_btn_tic(2) <= '0';
			if v_button(2)='1' and v_btn_old(2)='0' then
				v_btn_old(2) <= '1';
				v_btn_tic(2) <= '1';
			elsif v_button(2)='0' and v_btn_old(2)='1' then
				v_btn_old(2) <= '0';
			end if;			
			-- west
			v_btn_tic(3) <= '0';
			if v_button(3)='1' and v_btn_old(3)='0' then
				v_btn_old(3) <= '1';
				v_btn_tic(3) <= '1';
			elsif v_button(3)='0' and v_btn_old(3)='1' then
				v_btn_old(3) <= '0';
			end if;			
		end if;
	end process P_Buttons;	
	
	taster_out <= v_btn_tic;

end Verhalten;

